﻿using System;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Threading;
using ThinkGeo.MapSuite.SilverlightCore;
using ThinkGeo.MapSuite.SilverlightEdition;

namespace FeatureInfo
{
    public partial class MainPage : UserControl
    {
        private PointShape mouseWorldPosition;
        private DispatcherTimer hoverTimer;

        public MainPage()
        {
            InitializeComponent();

            // This timer saves performance a lot while mouse hovering.
            hoverTimer = new DispatcherTimer();
            hoverTimer.Interval = TimeSpan.FromMilliseconds(300);
            hoverTimer.Tick += new System.EventHandler(hoverTimer_Tick);
        }

        private void Map1_Loaded(object sender, RoutedEventArgs e)
        {
            Map1.CurrentExtent = new RectangleShape(-132, 55, -70, 20);
            Map1.MapUnit = GeographyUnit.DecimalDegree;

            WorldMapKitWmsSilverlightOverlay baseOverlay = new WorldMapKitWmsSilverlightOverlay();
            Map1.Overlays.Add(baseOverlay);

            LayerOverlay featureOverlay = new LayerOverlay();
            Map1.Overlays.Add("FeatureOverlay", featureOverlay);

            InMemoryFeatureLayer featureLayer = new InMemoryFeatureLayer();
            featureLayer.Columns.Add(new FeatureSourceColumn("Tips"));
            featureLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateLinearGradientStyle(new Vertex(), new Vertex(1, 1), GeoColor.FromArgb(100, GeoColor.FromHtml("#F46F50")), GeoColor.FromArgb(100, GeoColor.FromHtml("#DF4124")), GeoColor.FromHtml("#EB451F"));
            featureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            featureOverlay.Layers.Add("FeatureLayer", featureLayer);
            InitializeShapes(featureLayer);

            Map1.Refresh();
        }

        // This method initialize the features.
        private void InitializeShapes(InMemoryFeatureLayer shapeLayer)
        {
            BaseShape areaShape1 = new RectangleShape(-110, 40, -100, 30);
            BaseShape areaShape2 = new RectangleShape(-105, 45, -95, 35);

            Feature feature1 = new Feature(areaShape1);
            feature1.ColumnValues.Add("Tips", "AreaShape1");

            Feature feature2 = new Feature(areaShape2);
            feature2.ColumnValues.Add("Tips", "AreaShape2");

            shapeLayer.InternalFeatures.Add("AreaShape1", feature1);
            shapeLayer.InternalFeatures.Add("AreaShape2", feature2);
        }

        // Use this event to implement auto size.
        private void UserControl_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            Dispatcher.BeginInvoke(() => {
                if (Map1 != null)
                {
                    Map1.Width = e.NewSize.Width;
                    Map1.Height = e.NewSize.Height;
                }
            });
        }

        private void Map1_MouseMove(object sender, System.Windows.Input.MouseEventArgs e)
        {
            if (hoverTimer.IsEnabled) { hoverTimer.Stop(); }

            Point screenPoint = e.GetPosition(Map1);
            mouseWorldPosition = Map1.ToWorldCoordinate(screenPoint);
            hoverTimer.Start();
        }

        private void hoverTimer_Tick(object sender, System.EventArgs e)
        {
            LayerOverlay featureOverlay = (LayerOverlay)Map1.Overlays["FeatureOverlay"];
            InMemoryFeatureLayer featureLayer = (InMemoryFeatureLayer)featureOverlay.Layers["FeatureLayer"];

            featureLayer.Open();
            Collection<Feature> queriedFeatures = featureLayer.QueryTools.GetFeaturesContaining(mouseWorldPosition, ReturningColumnsType.AllColumns);
            featureLayer.Close();

            if (queriedFeatures.Count > 0)
            {
                if (Map1.Popups.Count != 0) { Map1.Popups.Clear(); }

                Feature feature = queriedFeatures[0];
                string content = String.Format(CultureInfo.InvariantCulture, "Tips:{0}", feature.ColumnValues["Tips"]);
                GeoPopup popup = GetPopup(content, mouseWorldPosition);
                Map1.Popups.Add(popup);
            }
            else if (Map1.Popups.Count != 0) { Map1.Popups.Clear(); }

            hoverTimer.Stop();
        }

        private static GeoPopup GetPopup(string content, PointShape position)
        {
            TextBlock textBlock = new TextBlock();
            textBlock.Text = content;
            textBlock.Width = 100;
            textBlock.TextWrapping = TextWrapping.Wrap;

            GeoPopup popup = new GeoPopup(position);
            popup.Width = 120;
            popup.Height = 40;
            popup.Offset = new Point(-80, -20);
            popup.Content = textBlock;

            return popup;
        }
    }
}
